let request = require('request');

let curlCommands = [];

exports.getCurlCommands = function(req, res) {
    let id = getSessionId(req);
    if (!id) {
        res.send([]);
        return;
    }
    if (!curlCommands[id]) {
        res.send([]);
        return;
    }
    res.send(curlCommands[id]);
};

exports.get = async function(req, context, disableFollowRedirect = false) {
    return await doGetRequest(req, context, disableFollowRedirect);
};

exports.post = async function(req, context, disableFollowRedirect = false) {
    return await doPostRequest(req, context, disableFollowRedirect);
};

exports.put = async function(req, context, disableFollowRedirect = false) {
    return await doPutRequest(req, context, disableFollowRedirect);
};


exports.delete = async function(req, context, disableFollowRedirect = false) {
    return await doDeleteRequest(req, context, disableFollowRedirect);
};


let doPostRequest = function (req, context, disableFollowRedirect) {
    if (disableFollowRedirect) {
        context.followRedirect = false;
    }
    return new Promise(function (resolve, reject) {
        generateCurlCommand(req,"post", context);
        request.post(context, function (error, response, body) {
            if (!error) {
                if (response.statusCode === 301 && disableFollowRedirect) {
                    context.url = response.headers.location;
                    console.log("Redirect to " + context.url);
                    doPostRequest(context).then((response, error) => {
                        console.log(response);
                        if (error) {
                            reject(error);
                        } else {
                            resolve({statusCode: response.statusCode, body: response.body});
                        }
                    });
                } else {
                    resolve({statusCode: response.statusCode, body: response.body});
                }
            } else {
                reject(error);
            }
        });
    });
};

let doPutRequest = function (req, context, disableFollowRedirect) {
    if (disableFollowRedirect) {
        context.followRedirect = false;
    }
    return new Promise(function (resolve, reject) {
        let curlCommand = generateCurlCommand(req, "put", context);
        request.put(context, function (error, response, body) {
            if (!error) {
                if (response.statusCode === 301 && disableFollowRedirect) {
                    context.url = response.headers.location;
                    console.log("Redirect to " + context.url);
                    doPostRequest(context).then((response, error) => {
                        console.log(response);
                        if (error) {
                            reject(error);
                        } else {
                            resolve({statusCode: response.statusCode, body: response.body});
                        }
                    });
                } else {
                    resolve({statusCode: response.statusCode, body: response.body});
                }
            } else {
                reject(error);
            }
        });
    });
};


let doGetRequest = function (req, context, disableFollowRedirect) {
    if (disableFollowRedirect) {
        context.followRedirect = false;
    }

    return new Promise(function (resolve, reject) {

        let curlCommand = generateCurlCommand(req, "get", context);
        request.get(context, function (error, response, body) {
            if (!error) {
                if (response.statusCode === 301 && disableFollowRedirect) {
                    context.url = response.headers.location;
                    console.log("Redirect to " + context.url);
                    doGetRequest(context, disableFollowRedirect).then((response, error) => {
                        console.log(response);
                        if (error) {
                            reject(error);
                        } else {
                            resolve({statusCode: response.statusCode, body: response.body});
                        }
                    });
                } else {
                    resolve({statusCode: response.statusCode, body: response.body});
                }
            } else {
                reject(error);
            }
        });
    });
};

let doDeleteRequest = function (req, context, disableFollowRedirect) {
    if (disableFollowRedirect) {
        context.followRedirect = false;
    }

    return new Promise(function (resolve, reject) {

        let curlCommand = generateCurlCommand(req, "delete", context);
        request.delete(context, function (error, response, body) {
            if (!error) {
                if (response.statusCode === 301 && disableFollowRedirect) {
                    context.url = response.headers.location;
                    console.log("Redirect to " + context.url);
                    doGetRequest(context, disableFollowRedirect).then((response, error) => {
                        console.log(response);
                        if (error) {
                            reject(error);
                        } else {
                            resolve({statusCode: response.statusCode, body: response.body});
                        }
                    });
                } else {
                    resolve({statusCode: response.statusCode, body: response.body});
                }
            } else {
                reject(error);
            }
        });
    });
};

let generateCurlCommand = function (req, method, context) {
    let curlCommand = "curl ";

    if (context.headers) {
        for(let propt in context.headers){
            curlCommand += "-H \"" + propt + ": " + context.headers[propt] + "\" ";
        }

    }
    if (context.body) {
        curlCommand += "-d '" + context.body + "' "
    }
    switch (method) {
        case "post":
            curlCommand += "-X POST ";
            break;
        case "delete":
            curlCommand += "-X DELETE ";
            break;
        case "put":
            curlCommand += "-X PUT ";
            break;
    }

    curlCommand += "\"" + context.url + "\"";

    addCurlCommand(req, curlCommand);
};

let addCurlCommand = function (req, curlCommand) {
    let id = getSessionId(req);
    if (!id) {
        return;
    }

    if (!curlCommands[id]) {
        curlCommands[id] = [];
    }
    curlCommands[id].splice(0, 0, curlCommand);
    if (curlCommands[id].length > 10) {
        curlCommands[id].pop();
    }

};

let getSessionId = function (req) {
    if (req.body && req.body.sessionId) {
        return req.body.sessionId;
    }

    if (req.query && req.query.sessionId) {
        return req.query.sessionId;
    }
};